@@ -1,28 +1,25 @@
 #include "fps_counter.h"
 
 #include <assert.h>
-#include <SDL2/SDL_timer.h>
 
-#include "config.h"
-#include "util/lock.h"
 #include "util/log.h"
 
-#define FPS_COUNTER_INTERVAL_MS 1000
+#define FPS_COUNTER_INTERVAL SC_TICK_FROM_SEC(1)
 
 bool
 fps_counter_init(struct fps_counter *counter) {
-    counter->mutex = SDL_CreateMutex();
-    if (!counter->mutex) {
+    bool ok = sc_mutex_init(&counter->mutex);
+    if (!ok) {
         return false;
     }
 
-    counter->state_cond = SDL_CreateCond();
-    if (!counter->state_cond) {
-        SDL_DestroyMutex(counter->mutex);
+    ok = sc_cond_init(&counter->state_cond);
+    if (!ok) {
+        sc_mutex_destroy(&counter->mutex);
         return false;
     }
 
-    counter->thread = NULL;
+    counter->thread_started = false;
     atomic_init(&counter->started, 0);
     // no need to initialize the other fields, they are unused until started
 
@@ -31,8 +28,8 @@ fps_counter_init(struct fps_counter *counter) {
 
 void
 fps_counter_destroy(struct fps_counter *counter) {
-    SDL_DestroyCond(counter->state_cond);
-    SDL_DestroyMutex(counter->mutex);
+    sc_cond_destroy(&counter->state_cond);
+    sc_mutex_destroy(&counter->mutex);
 }
 
 static inline bool
@@ -49,7 +46,7 @@ set_started(struct fps_counter *counter, bool started) {
 static void
 display_fps(struct fps_counter *counter) {
     unsigned rendered_per_second =
-        counter->nr_rendered * 1000 / FPS_COUNTER_INTERVAL_MS;
+        counter->nr_rendered * SC_TICK_FREQ / FPS_COUNTER_INTERVAL;
     if (counter->nr_skipped) {
         LOGI("%u fps (+%u frames skipped)", rendered_per_second,
                                             counter->nr_skipped);
@@ -70,53 +67,54 @@ check_interval_expired(struct fps_counter *counter, uint32_t now) {
     counter->nr_skipped = 0;
     // add a multiple of the interval
     uint32_t elapsed_slices =
-        (now - counter->next_timestamp) / FPS_COUNTER_INTERVAL_MS + 1;
-    counter->next_timestamp += FPS_COUNTER_INTERVAL_MS * elapsed_slices;
+        (now - counter->next_timestamp) / FPS_COUNTER_INTERVAL + 1;
+    counter->next_timestamp += FPS_COUNTER_INTERVAL * elapsed_slices;
 }
 
 static int
 run_fps_counter(void *data) {
     struct fps_counter *counter = data;
 
-    mutex_lock(counter->mutex);
+    sc_mutex_lock(&counter->mutex);
     while (!counter->interrupted) {
         while (!counter->interrupted && !is_started(counter)) {
-            cond_wait(counter->state_cond, counter->mutex);
+            sc_cond_wait(&counter->state_cond, &counter->mutex);
         }
         while (!counter->interrupted && is_started(counter)) {
-            uint32_t now = SDL_GetTicks();
+            sc_tick now = sc_tick_now();
             check_interval_expired(counter, now);
 
-            assert(counter->next_timestamp > now);
-            uint32_t remaining = counter->next_timestamp - now;
-
             // ignore the reason (timeout or signaled), we just loop anyway
-            cond_wait_timeout(counter->state_cond, counter->mutex, remaining);
+            sc_cond_timedwait(&counter->state_cond, &counter->mutex,
+                              counter->next_timestamp);
         }
     }
-    mutex_unlock(counter->mutex);
+    sc_mutex_unlock(&counter->mutex);
     return 0;
 }
 
 bool
 fps_counter_start(struct fps_counter *counter) {
-    mutex_lock(counter->mutex);
-    counter->next_timestamp = SDL_GetTicks() + FPS_COUNTER_INTERVAL_MS;
+    sc_mutex_lock(&counter->mutex);
+    counter->next_timestamp = sc_tick_now() + FPS_COUNTER_INTERVAL;
     counter->nr_rendered = 0;
     counter->nr_skipped = 0;
-    mutex_unlock(counter->mutex);
+    sc_mutex_unlock(&counter->mutex);
 
     set_started(counter, true);
-    cond_signal(counter->state_cond);
+    sc_cond_signal(&counter->state_cond);
 
-    // counter->thread is always accessed from the same thread, no need to lock
-    if (!counter->thread) {
-        counter->thread =
-            SDL_CreateThread(run_fps_counter, "fps counter", counter);
-        if (!counter->thread) {
+    // counter->thread_started and counter->thread are always accessed from the
+    // same thread, no need to lock
+    if (!counter->thread_started) {
+        bool ok = sc_thread_create(&counter->thread, run_fps_counter,
+                                   "fps counter", counter);
+        if (!ok) {
             LOGE("Could not start FPS counter thread");
             return false;
         }
+
+        counter->thread_started = true;
     }
 
     return true;
@@ -125,7 +123,7 @@ fps_counter_start(struct fps_counter *counter) {
 void
 fps_counter_stop(struct fps_counter *counter) {
     set_started(counter, false);
-    cond_signal(counter->state_cond);
+    sc_cond_signal(&counter->state_cond);
 }
 
 bool
@@ -135,21 +133,25 @@ fps_counter_is_started(struct fps_counter *counter) {
 
 void
 fps_counter_interrupt(struct fps_counter *counter) {
-    if (!counter->thread) {
+    if (!counter->thread_started) {
         return;
     }
 
-    mutex_lock(counter->mutex);
+    sc_mutex_lock(&counter->mutex);
     counter->interrupted = true;
-    mutex_unlock(counter->mutex);
+    sc_mutex_unlock(&counter->mutex);
     // wake up blocking wait
-    cond_signal(counter->state_cond);
+    sc_cond_signal(&counter->state_cond);
 }
 
 void
 fps_counter_join(struct fps_counter *counter) {
-    if (counter->thread) {
-        SDL_WaitThread(counter->thread, NULL);
+    if (counter->thread_started) {
+        // interrupted must be set by the thread calling join(), so no need to
+        // lock for the assertion
+        assert(counter->interrupted);
+
+        sc_thread_join(&counter->thread, NULL);
     }
 }
 
@@ -159,11 +161,11 @@ fps_counter_add_rendered_frame(struct fps_counter *counter) {
         return;
     }
 
-    mutex_lock(counter->mutex);
-    uint32_t now = SDL_GetTicks();
+    sc_mutex_lock(&counter->mutex);
+    sc_tick now = sc_tick_now();
     check_interval_expired(counter, now);
     ++counter->nr_rendered;
-    mutex_unlock(counter->mutex);
+    sc_mutex_unlock(&counter->mutex);
 }
 
 void
@@ -172,9 +174,9 @@ fps_counter_add_skipped_frame(struct fps_counter *counter) {
         return;
     }
 
-    mutex_lock(counter->mutex);
-    uint32_t now = SDL_GetTicks();
+    sc_mutex_lock(&counter->mutex);
+    sc_tick now = sc_tick_now();
     check_interval_expired(counter, now);
     ++counter->nr_skipped;
-    mutex_unlock(counter->mutex);
+    sc_mutex_unlock(&counter->mutex);
 }
