@@ -1,29 +1,39 @@
 #ifndef SCREEN_H
 #define SCREEN_H
 
+#include "common.h"
+
 #include <stdbool.h>
 #include <SDL2/SDL.h>
 #include <libavformat/avformat.h>
 
-#include "config.h"
-#include "common.h"
+#include "coords.h"
+#include "fps_counter.h"
 #include "opengl.h"
-
-struct video_buffer;
+#include "trait/frame_sink.h"
+#include "video_buffer.h"
 
 struct screen {
+    struct sc_frame_sink frame_sink; // frame sink trait
+
+#ifndef NDEBUG
+    bool open; // track the open/close state to assert correct behavior
+#endif
+
+    struct sc_video_buffer vb;
+    struct fps_counter fps_counter;
+
     SDL_Window *window;
     SDL_Renderer *renderer;
     SDL_Texture *texture;
-    bool use_opengl;
     struct sc_opengl gl;
-    struct size frame_size;
-    struct size content_size; // rotated frame_size
+    struct sc_size frame_size;
+    struct sc_size content_size; // rotated frame_size
 
     bool resize_pending; // resize requested while fullscreen or maximized
     // The content size the last time the window was not maximized or
     // fullscreen (meaningful only when resize_pending is true)
-    struct size windowed_content_size;
+    struct sc_size windowed_content_size;
 
     // client rotation: 0, 1, 2 or 3 (x90 degrees counterclockwise)
     unsigned rotation;
@@ -32,74 +42,56 @@ struct screen {
     bool has_frame;
     bool fullscreen;
     bool maximized;
-    bool no_window;
     bool mipmaps;
+
+    bool event_failed; // in case SDL_PushEvent() returned an error
+
+    AVFrame *frame;
 };
 
-#define SCREEN_INITIALIZER { \
-    .window = NULL, \
-    .renderer = NULL, \
-    .texture = NULL, \
-    .use_opengl = false, \
-    .gl = {0}, \
-    .frame_size = { \
-        .width = 0, \
-        .height = 0, \
-    }, \
-    .content_size = { \
-        .width = 0, \
-        .height = 0, \
-    }, \
-    .resize_pending = false, \
-    .windowed_content_size = { \
-        .width = 0, \
-        .height = 0, \
-    }, \
-    .rotation = 0, \
-    .rect = { \
-        .x = 0, \
-        .y = 0, \
-        .w = 0, \
-        .h = 0, \
-    }, \
-    .has_frame = false, \
-    .fullscreen = false, \
-    .maximized = false, \
-    .no_window = false, \
-    .mipmaps = false, \
-}
+struct screen_params {
+    const char *window_title;
+    struct sc_size frame_size;
+    bool always_on_top;
 
-// initialize default values
-void
-screen_init(struct screen *screen);
+    int16_t window_x;
+    int16_t window_y;
+    uint16_t window_width;  // accepts SC_WINDOW_POSITION_UNDEFINED
+    uint16_t window_height; // accepts SC_WINDOW_POSITION_UNDEFINED
+
+    bool window_borderless;
+
+    uint8_t rotation;
+    bool mipmaps;
+
+    bool fullscreen;
+
+    sc_tick buffering_time;
+};
 
 // initialize screen, create window, renderer and texture (window is hidden)
-// window_x and window_y accept SC_WINDOW_POSITION_UNDEFINED
 bool
-screen_init_rendering(struct screen *screen, const char *window_title,
-                      struct size frame_size, bool always_on_top,
-                      int16_t window_x, int16_t window_y, uint16_t window_width,
-                      uint16_t window_height, bool window_borderless,
-                      uint8_t rotation, bool mipmaps);
+screen_init(struct screen *screen, const struct screen_params *params);
 
-// show the window
+// request to interrupt any inner thread
+// must be called before screen_join()
 void
-screen_show_window(struct screen *screen);
+screen_interrupt(struct screen *screen);
+
+// join any inner thread
+void
+screen_join(struct screen *screen);
 
 // destroy window, renderer and texture (if any)
 void
 screen_destroy(struct screen *screen);
 
-// resize if necessary and write the rendered frame into the texture
-bool
-screen_update_frame(struct screen *screen, struct video_buffer *vb);
-
-// render the texture to the renderer
+// hide the window
 //
-// Set the update_content_rect flag if the window or content size may have
-// changed, so that the content rectangle is recomputed
+// It is used to hide the window immediately on closing without waiting for
+// screen_destroy()
 void
-screen_render(struct screen *screen, bool update_content_rect);
+screen_hide_window(struct screen *screen);
 
 // switch the fullscreen mode
 void
@@ -117,19 +109,19 @@ screen_resize_to_pixel_perfect(struct screen *screen);
 void
 screen_set_rotation(struct screen *screen, unsigned rotation);
 
-// react to window events
-void
-screen_handle_window_event(struct screen *screen, const SDL_WindowEvent *event);
+// react to SDL events
+bool
+screen_handle_event(struct screen *screen, SDL_Event *event);
 
 // convert point from window coordinates to frame coordinates
 // x and y are expressed in pixels
-struct point
+struct sc_point
 screen_convert_window_to_frame_coords(struct screen *screen,
                                       int32_t x, int32_t y);
 
 // convert point from drawable coordinates to frame coordinates
 // x and y are expressed in pixels
-struct point
+struct sc_point
 screen_convert_drawable_to_frame_coords(struct screen *screen,
                                         int32_t x, int32_t y);
 
