@@ -12,15 +12,17 @@
 set -e
 
 SCRCPY_DEBUG=false
-SCRCPY_VERSION_NAME=1.19
+SCRCPY_VERSION_NAME=1.21
 
-PLATFORM=${ANDROID_PLATFORM:-30}
-BUILD_TOOLS=${ANDROID_BUILD_TOOLS:-30.0.0}
+PLATFORM_VERSION=31
+PLATFORM=${ANDROID_PLATFORM:-$PLATFORM_VERSION}
+BUILD_TOOLS=${ANDROID_BUILD_TOOLS:-31.0.0}
 
 BUILD_DIR="$(realpath ${BUILD_DIR:-build_manual})"
 CLASSES_DIR="$BUILD_DIR/classes"
 SERVER_DIR=$(dirname "$0")
 SERVER_BINARY=scrcpy-server
+ANDROID_JAR="$ANDROID_HOME/platforms/android-$PLATFORM/android.jar"
 
 echo "Platform: android-$PLATFORM"
 echo "Build-tools: $BUILD_TOOLS"
@@ -47,23 +49,40 @@ cd "$SERVER_DIR/src/main/aidl"
 
 echo "Compiling java sources..."
 cd ../java
-javac -bootclasspath "$ANDROID_HOME/platforms/android-$PLATFORM/android.jar" \
-    -cp "$CLASSES_DIR" -d "$CLASSES_DIR" -source 1.8 -target 1.8 \
+javac -bootclasspath "$ANDROID_JAR" -cp "$CLASSES_DIR" -d "$CLASSES_DIR" \
+    -source 1.8 -target 1.8 \
     com/genymobile/scrcpy/*.java \
     com/genymobile/scrcpy/wrappers/*.java
 
 echo "Dexing..."
 cd "$CLASSES_DIR"
-"$ANDROID_HOME/build-tools/$BUILD_TOOLS/dx" --dex \
-    --output "$BUILD_DIR/classes.dex" \
-    android/view/*.class \
-    android/content/*.class \
-    com/genymobile/scrcpy/*.class \
-    com/genymobile/scrcpy/wrappers/*.class
 
-echo "Archiving..."
-cd "$BUILD_DIR"
-jar cvf "$SERVER_BINARY" classes.dex
-rm -rf classes.dex classes
+if [[ $PLATFORM_VERSION -lt 31 ]]
+then
+    # use dx
+    "$ANDROID_HOME/build-tools/$BUILD_TOOLS/dx" --dex \
+        --output "$BUILD_DIR/classes.dex" \
+        android/view/*.class \
+        android/content/*.class \
+        com/genymobile/scrcpy/*.class \
+        com/genymobile/scrcpy/wrappers/*.class
+
+    echo "Archiving..."
+    cd "$BUILD_DIR"
+    jar cvf "$SERVER_BINARY" classes.dex
+    rm -rf classes.dex classes
+else
+    # use d8
+    "$ANDROID_HOME/build-tools/$BUILD_TOOLS/d8" --classpath "$ANDROID_JAR" \
+        --output "$BUILD_DIR/classes.zip" \
+        android/view/*.class \
+        android/content/*.class \
+        com/genymobile/scrcpy/*.class \
+        com/genymobile/scrcpy/wrappers/*.class
+
+    cd "$BUILD_DIR"
+    mv classes.zip "$SERVER_BINARY"
+    rm -rf classes
+fi
 
 echo "Server generated in $BUILD_DIR/$SERVER_BINARY"
